Change Log
==========

All notable changes to the project are documented in this file.

[v1.8] - 2015-02-02
-------------------

### Changes
- Updated [README.md]
- Add [TODO.md]
- Add [CHANGELOG.md], attempt to align with http://keepachangelog.com
- From now on Travis CI only runs when pushing to the dev branch,
  so all new development must be done there.

### Fixes
- Fix insecure `chroot()` reported in [Coverity Scan] CID #54523.
- Minor cleanup fixes.


[v1.7] - 2014-12-21
-------------------

The TFTP Blocksize Negotiation release.

### Changes
- Support for [RFC 2348], TFTP blocksize negotiation
- Support for custom server directory, instead of FTP user's `$HOME`
- Log to `stderr` when running in foreground and debug is enabled


[v1.6] - 2014-09-12
-------------------

Fix missing [libuEv] directory content generated by <kbd>make dist</kbd>
in [v1.3], [v1.4], and [v1.5].

### Fixes
- Since the introduction of the event library [libuEv] the <kbd>make
  dist</kbd> target has failed to include the libuev sub-directory.
  This is due to the `git archive` command unfortunately not supporting
  git sub-modules.


[v1.5] - 2014-09-12 [YANKED]
----------------------------

Major fix release, lots of issues reported by [Coverity Scan] fixed.
For details, see the analysis at https://scan.coverity.com/projects/2947

**Note:** This release has been *yanked* from distribution due to the
tarball (generated by the <kbd>make dist</kbd>) missing the required
libuEv library.  Instead, use [v1.6] or later, where this is fixed, or
roll your own build of this release from the GIT source tree.

### Changes
- Add support for [Travis-CI], continuous integration with GitHub
- Add support for [Coverity Scan], the best static code analyzer,
  integrated with [Travis-CI] -- scan runs for each push to master

### Fixes
- Fix nasty invalid `sizeof()` argument to `recv()` causing uftpd to
  only read 4/8 bytes (32/64 bit arch) at a time from the FTP socket.
  This should greatly reduce CPU utilisation and improve xfer speeds.
  Found by [Coverity Scan].
- Fix minor resource leak in `ftp_session()` when `getsockname()` or
  `getpeername()` fail.  Minor fix because the session exits and the OS
  usually frees resources at that point, unless you're using uClinux.
  Found by [Coverity Scan].
- Various fixes for unchecked API return values, prevents propagation of
  errors.  Also, make sure to clear input data before calling API's.
  Found by [Coverity Scan].
- Fix oversight in checking for invalid/missing FTP username.
  Found by [Coverity Scan].
- Fix potential attack vector.  Make sure to always store a NUL string
  terminator in all received FTP commands so the parser does not go out
  of bounds. Found by [Coverity Scan].
- Fix parallel build problems in `Makefile`.


[v1.4] - 2014-09-04 [YANKED]
----------------------------

**Note:** This release has been *yanked* from distribution due to the
tarball (generated by the <kbd>make dist</kbd>) missing the required
libuEv library.  Instead, use [v1.6] or later, where this is fixed, or
roll your own build of this release from the GIT source tree.

### Changes
- Update documentation, both built-in usage text and man page.

### Fixes
- Fix bug in inetd.conf installed by .deb package for TFTP service.
  Inetd forked off a new TFTP session for each connection attempt.


[v1.3] - 2014-09-04 [YANKED]
----------------------------

Added support for TFTP, [RFC 1350].  Integration of the asynchronous
event library [libuEv], to serialize all events.  Massive refactoring.

**Note:** This release has been *yanked* from distribution due to the
tarball (generated by the <kbd>make dist</kbd>) missing the required
libuEv library.  Instead, use [v1.6] or later, where this is fixed, or
roll your own build of this release from the GIT source tree.

### Changes
- Incompatible changes to the command line arguments, compared to v1.2!
- Add libuEv as a GIT submodule, handles signals, timers, and all I/O.
- Refactor all signal handling, timers, and socket `poll()` calls to
  use libuEv instead.  Much cleaner and maintaiable code as a result.
- Clarify copyright claims, not much remains of the original [FtpServer]
  code, by [Xu Wang].


[v1.2] - 2014-05-19
-------------------

### Changes
- Add support for logging to stdout as well as syslog.

### Fixes
- Fix embarrassing problem with listing big/average sized directories.


[v1.1] - 2014-05-04
-------------------

Haunted zombie (¬°-°)¬ release.

### Changes
- Add strict FTP session inactivity timer, 20 sec.
- Change some logs to informational, only seen in verbose `-V` mode.
- Revise .deb package slightly and add support for creating an FTP user
  and group on the system.  This is used to both find the default FTP
  home directory, to serve files from, and also the UID/GID to drop to
  when being started as root.

### Fixes
- Fix zombie problem.  Forked off FTP sessions did not exit properly and
  were not `wait()`'ed for properly, so uftpd left a zombie processes
  lingering after each session.
- Fix ordering bug in security mechanism "drop privs"


[v1.0] - 2014-05-04
-------------------

First official uftpd release! :-)

### Changes
- Forked from [FtpServer], by [Xu Wang].
- Add permissive [ISC license].
- Massive refactor, code cleanup/renaming and "UNIX'ification":
  - Add actual command line parser.
  - Cleanup all log messages.
  - Reindent to use Linux KNF.
  - Use system's FTP user to figure out FTP home directory, with
    built-in fallback to `/srv/ftp`
  - Use system's `ftp/tcp` port from `/etc/services`.
  - Chroot to FTP home directory.
  - Support for dropping privileges if a valid FTP user exists.
  - Use `fork()` instead of pthreads for FTP client sessions.
  - Daemonize uftpd by default, detach from controlling terminal and
    reparent to PID 1 (init).
  - Add support for running as an `inetd` service.
  - Add wrapper for `syslog()` instead of using `stdout/stderr`.
  - Add basic `uftpd.8` man page.
- Add OpenBSD `strlcat()` and `strlcpy()` safe string functions.
- Add support for NOOP (keepalive sent by some clients).
- Add support for SIZE.
- Add support for TYPE, at least `IMAGE/BINARY`.
- Add basic dependency handling to Makefile.
- Add support for building Debian .deb packages.

### Fixes
- Handle "walking up to parent" attacks in several FTP functions.
- Fix memory leaks in `recv_mesg()` caused by dangerous homegrown string
  functions.  Replaced with safer OpenBSD variants.
- Fix absolute paths in FTP `LIST` command.
- Fix Firefox FTP mode `LIST` compatibility issue.
- Fix "bare linefeeds" warning from certain FTP clients in ASCII mode.
  Lines must end in the old `\r\n` format, rather than UNIX `\n`.


[UNRELEASED]:    https://github.com/troglobit/uftpd/compare/v1.8...HEAD
[v1.8]:          https://github.com/troglobit/uftpd/compare/v1.7...v1.8
[v1.7]:          https://github.com/troglobit/uftpd/compare/v1.6...v1.7
[v1.6]:          https://github.com/troglobit/uftpd/compare/v1.5...v1.6
[v1.5]:          https://github.com/troglobit/uftpd/compare/v1.4...v1.5
[v1.4]:          https://github.com/troglobit/uftpd/compare/v1.3...v1.4
[v1.3]:          https://github.com/troglobit/uftpd/compare/v1.2...v1.3
[v1.2]:          https://github.com/troglobit/uftpd/compare/v1.1...v1.2
[v1.1]:          https://github.com/troglobit/uftpd/compare/v1.0...v1.1
[libuEv]:        http://github.com/troglobit/libuev
[ISC license]:   http://en.wikipedia.org/wiki/ISC_license
[RFC 1350]:      http://tools.ietf.org/html/rfc1350
[RFC 2348]:      http://tools.ietf.org/html/rfc2348
[Xu Wang]:       https://github.com/xu-wang11/
[FtpServer]:     https://github.com/xu-wang11/FtpServer
[Travis-CI]:     https://travis-ci.org/troglobit/uftpd
[Coverity Scan]: https://scan.coverity.com/projects/2947
[TODO.md]:       https://github.com/troglobit/uftpd/blob/master/TODO.md
[README.md]:     https://github.com/troglobit/uftpd/blob/master/README.md
[CHANGELOG.md]:  https://github.com/troglobit/uftpd/blob/master/CHANGELOG.md
